package eva2.server.go.operators.moso;

import eva2.server.go.individuals.AbstractEAIndividual;
import eva2.server.go.populations.Population;
import eva2.tools.math.RNG;

/**
 * Created by IntelliJ IDEA.
 * User: streiche
 * Date: 30.03.2004
 * Time: 17:56:39
 * To change this template use File | Settings | File Templates.
 */
public class MOSORandomChoice  implements InterfaceMOSOConverter, java.io.Serializable {

    private int             m_OutputDimension   = 2;

    public MOSORandomChoice() {
    }
    public MOSORandomChoice(MOSORandomChoice b) {
        this.m_OutputDimension  = b.m_OutputDimension;
    }
    public Object clone() {
        return (Object) new MOSORandomChoice(this);
    }

    /** This method takes a population of individuals with an array of
     * fitness values and calculates a single fitness value to replace
     * the former fitness array. Please note: The orignal fitness values
     * are lost this way, so please use the individual.setData() method
     * if you still want to access the original fitness values.
     * @param pop       The population to process.
     */
    public void convertMultiObjective2SingleObjective(Population pop) {
        for (int i = 0; i < pop.size(); i++) {
             this.convertSingleIndividual((AbstractEAIndividual)pop.get(i));
        }
    }

    /** This method processes a single individual
     * @param indy      The individual to process.
     */
    public void convertSingleIndividual(AbstractEAIndividual indy) {
        double[]    resultFit = new double[1];
        double[]    tmpFit;

        tmpFit          = indy.getFitness();
        indy.putData("MOFitness", tmpFit);
        resultFit[0]    = tmpFit[RNG.randomInt(0, tmpFit.length-1)];
        indy.SetFitness(resultFit);
    }

    /** This method allows the problem to set the current output size of
     * the optimization problem. Additional weights will be set to a default
     * value of one
     * @param dim       Outputdimension of the problem
     */
    public void setOutputDimension(int dim) {
        this.m_OutputDimension = dim;
        // i think as far as i got not solution for the (n>2) dimensional case
        // i could simply ignore this....
    }
    
    /** This method returns a description of the objective
     * @return A String
     */
    public String getStringRepresentation() {
        return this.getName()+"\n";
    }



/**********************************************************************************************************************
 * These are for GUI
 */
    /** This method allows the CommonJavaObjectEditorPanel to read the
     * name to the current object.
     * @return The name.
     */
    public String getName() {
        return "Random Choice";
    }

    /** This method returns a global info string
     * @return description
     */
    public static String globalInfo() {
        return "This method selects a random fitness value, actually this implements VEGA [Schaffer84Experiments].";
    }
}